@@ -27,7 +27,6 @@
 #' saveWorkbook(wb, "loadExample.xlsx", overwrite = TRUE)
 loadWorkbook <- function(xlsxFile){
   
-  
   if(!file.exists(xlsxFile))
     stop("File does not exist.")
   
@@ -59,6 +58,13 @@ loadWorkbook <- function(xlsxFile){
   extLinksXML       <- xmlFiles[grepl("externalLink[0-9]+.xml$", xmlFiles, perl = TRUE)]
   extLinksRelsXML   <- xmlFiles[grepl("externalLink[0-9]+.xml.rels$", xmlFiles, perl = TRUE)]
   
+  # pivot tables
+  pivotTableXML     <- xmlFiles[grepl("pivotTable[0-9]+.xml$", xmlFiles, perl = TRUE)]
+  pivotTableRelsXML <- xmlFiles[grepl("pivotTable[0-9]+.xml.rels$", xmlFiles, perl = TRUE)]
+  pivotDefXML       <- xmlFiles[grepl("pivotCacheDefinition[0-9]+.xml$", xmlFiles, perl = TRUE)]
+  pivotDefRelsXML   <- xmlFiles[grepl("pivotCacheDefinition[0-9]+.xml.rels$", xmlFiles, perl = TRUE)]
+  pivotRecordsXML   <- xmlFiles[grepl("pivotCacheRecords[0-9]+.xml$", xmlFiles, perl = TRUE)]
+  
   
   nSheets <- length(worksheetsXML)
   
@@ -105,6 +111,51 @@ loadWorkbook <- function(xlsxFile){
     
     wb$sharedStrings <- vals
     
+  }
+  
+  ## xl\pivotTables & xl\pivotCache
+  if(length(pivotTableXML) > 0){
+    
+    nPivotTables      <- length(pivotTableXML)
+    rIds <- 10000L + 1:nPivotTables
+    
+    pivotTableXML     <- pivotTableXML[order(nchar(pivotTableXML), pivotTableXML)]
+    pivotTableRelsXML <- pivotTableRelsXML[order(nchar(pivotTableRelsXML), pivotTableRelsXML)]
+    pivotDefXML       <- pivotDefXML[order(nchar(pivotDefXML), pivotDefXML)]
+    pivotDefRelsXML   <- pivotDefRelsXML[order(nchar(pivotDefRelsXML), pivotDefRelsXML)]
+    pivotRecordsXML   <- pivotRecordsXML[order(nchar(pivotRecordsXML), pivotRecordsXML)]
+  
+    wb$pivotTables <- unlist(lapply(pivotTableXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"))))
+    wb$pivotTables.xml.rels <- unlist(lapply(pivotTableRelsXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"))))
+    wb$pivotDefinitions <- unlist(lapply(pivotDefXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"))))
+    wb$pivotDefinitionsRels <- unlist(lapply(pivotDefRelsXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"))))
+    wb$pivotRecords <- unlist(lapply(pivotRecordsXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"))))
+    
+    ## update content_types
+    wb$Content_Types <- c(wb$Content_Types, unlist(lapply(1:nPivotTables, contentTypePivotXML)))
+    
+    ## workbook rels
+    wb$workbook.xml.rels <- c(wb$workbook.xml.rels,    
+      sprintf('<Relationship Id="rId%s" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/pivotCacheDefinition" Target="pivotCache/pivotCacheDefinition%s.xml"/>', rIds, 1:nPivotTables)
+    )
+    
+    caches <- .Call("openxlsx_getChildlessNode", workbook, "<pivotCache ", PACKAGE = "openxlsx")
+    for(i in 1:length(caches))
+      caches[i] <- gsub('"rId[0-9]+"', sprintf('"rId%s"', rIds[i]), caches[i])
+    
+    wb$workbook$pivotCaches <- paste0('<pivotCaches>', paste(caches, collapse = ""), '</pivotCaches>')
+      
+    ## worksheet rels (which pivotTable belongs on which worksheet)
+    
+#     wb$worksheets_rels
+    
+#     wb$worksheets_rels
+#     <Relationship Id="rId1" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/pivotTable" Target="../pivotTables/pivotTable1.xml"/>
+      
+    
+#       <Relationship Id="rId99" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/pivotTable" Target="../pivotTables/pivotTable1.xml"/>
+      
+    
   }
   
   ## xl\styles
@@ -307,15 +358,6 @@ loadWorkbook <- function(xlsxFile){
   
   
   
-  
-  ## tables
-  if(length(tablesXML) > 0){
-    tablesXML <- tablesXML[order(nchar(tablesXML), tablesXML)]
-    wb$tables <- lapply(tablesXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx")))
-    wb$Content_Types <- c(wb$Content_Types, 
-                          sprintf('<Override PartName="/xl/tables/table%s.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.table+xml"/>', 1:length(wb$tables)+2))   
-  }
-  
   ## queryTables
   if(length(queryTablesXML) > 0){
     
@@ -443,9 +485,14 @@ loadWorkbook <- function(xlsxFile){
       if(!grepl("/>$", autoFilter))
         autoFilter <- gsub(">$", "/>", autoFilter)
       
-      wb$worksheets[[i]]$autoFilter <- autoFilter
+      wb$worksheets[[i]]$autoFilter <- autoFilter      
     }
     
+    ## tab colour (sheetPR)
+    tabColour <- .Call("openxlsx_getChildlessNode", wsData[[i]], "<tabColor ", PACKAGE = "openxlsx")
+    if(length(tabColour) > 0)
+      wb$worksheets[[i]]$sheetPr <- sprintf('<sheetPr>%s</sheetPr>', tabColour)
+    
     ## hyperlinks
     hyperlinks <- .Call("openxlsx_getChildlessNode", sheetData[[i]], "<hyperlink ", PACKAGE = "openxlsx")
     if(length(hyperlinks) > 0)
@@ -540,26 +587,47 @@ loadWorkbook <- function(xlsxFile){
     if(length(tablesXML) > 0){
       
       tables <- lapply(xml, function(x) as.integer(regmatches(x, regexpr("(?<=table)[0-9]+(?=\\.xml)", x, perl = TRUE))))
-      if(length(unlist(tables)) > 0){  
+      tableSheets <- unlist(lapply(1:length(sheetNumber), function(i) rep(sheetNumber[i], length(tables[[i]]))))  
+      
+      if(length(unlist(tables)) > 0){
         ## get the tables that belong to each worksheet and create a worksheets_rels for each
-        tCount <- 2L ## table r:Ids start at 2
+        tCount <- 2L ## table r:Ids start at 3
         for(i in 1:length(tables)){
-          k <- 1:length(tables[[i]]) + tCount
-          wb$worksheets_rels[[sheetNumber[i]]] <- unlist(c(wb$worksheets_rels[[sheetNumber[i]]],
-                                                           sprintf('<Relationship Id="rId%s" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/table" Target="../tables/table%s.xml"/>',  k,  k)))
-          
-          
-          wb$worksheets[[sheetNumber[i]]]$tableParts <- sprintf("<tablePart r:id=\"rId%s\"/>", k)
-          tCount <- tCount + length(k)
+          if(length(tables[[i]]) > 0){
+            k <- 1:length(tables[[i]]) + tCount
+            wb$worksheets_rels[[sheetNumber[i]]] <- unlist(c(wb$worksheets_rels[[sheetNumber[i]]],
+                                                             sprintf('<Relationship Id="rId%s" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/table" Target="../tables/table%s.xml"/>',  k,  k)))
+            
+            
+            wb$worksheets[[sheetNumber[i]]]$tableParts <- sprintf("<tablePart r:id=\"rId%s\"/>", k)
+            tCount <- tCount + length(k)
+          }
         }
         
-        wb$tables <- wb$tables[order(unlist(tables))]
+        names(tablesXML) <- basename(tablesXML)
+        tablesXML <-tablesXML[sprintf("table%s.xml", unlist(tables))]
+
+        wb$tables <- sapply(tablesXML, function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx")))
+        
+        ## pull out refs and attach names
+        refs <- regmatches(wb$tables, regexpr('(?<=ref=")[0-9A-Z:]+', wb$tables, perl = TRUE))
+        names(wb$tables) <- refs
+        
+        wb$Content_Types <- c(wb$Content_Types, sprintf('<Override PartName="/xl/tables/table%s.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.table+xml"/>', 1:length(wb$tables)+2))   
         
         ## relabel ids
         for(i in 1:length(wb$tables)){
           newId <- sprintf(' id="%s" ', i+2)
-          wb$tables[[i]] <- sub(' id="[0-9]+" ' ,newId, wb$tables[[i]])
+          wb$tables[[i]] <- sub(' id="[0-9]+" ' , newId, wb$tables[[i]])
         }
+        
+        displayNames <- unlist(regmatches(wb$tables, regexpr('(?<=displayName=").*?[^"]+', wb$tables, perl = TRUE)))
+        if(length(displayNames) != length(tablesXML))
+          displayNames <- paste0("Table", 1:length(tablesXML))
+        
+        attr(wb$tables, "sheet") <- tableSheets
+        attr(wb$tables, "tableName") <- displayNames
+        
       }
     } ## if(length(tablesXML) > 0)
     
@@ -623,7 +691,26 @@ loadWorkbook <- function(xlsxFile){
         
       }     
     }
-  } 
+  
+    ## pivot tables
+    if(length(pivotTableXML) > 0){
+      
+      pivotTableJ <- lapply(xml, function(x) as.integer(regmatches(x, regexpr("(?<=pivotTable)[0-9]+(?=\\.xml)", x, perl = TRUE))))
+      sheetWithPivot <- sheetNumber[which(sapply(pivotTableJ, length) > 0)]
+      pivotRels <- lapply(xml, function(x) {y <- x[grepl("pivotTable", x)]; y[order(nchar(y), y)]})
+      pivotRels <- pivotRels[sapply(pivotRels, length) > 0]
+      
+      ## Modify rIds
+      for(i in 1:length(pivotRels)){
+        for(j in 1:length(pivotRels[[i]]))  
+          pivotRels[[i]][j] <- gsub('"rId[0-9]+"', sprintf('"rId%s"', 10000L + j), pivotRels[[i]][j])
+        wb$worksheets_rels[[sheetWithPivot[i]]] <- c(wb$worksheets_rels[[sheetNumber[i]]], pivotRels[[i]])
+      }
+      
+    }
+  
+  
+  } ## end of worksheetRels
   
   ## table rels
   if(length(tableRelsXML) > 0){
