@@ -5,6 +5,7 @@
 #' @author Alexander Walker
 #' @param x object or a list of objects that can be handled by \code{\link{writeData}} to write to file
 #' @param file xlsx file name
+#' @param asTable write using writeDataTable as opposed to writeData
 #' @param ... optional parameters to pass to functions:
 #' \itemize{
 #'   \item{createWorkbook}
@@ -19,7 +20,7 @@
 #'   \item{\bold{creator}}{ A string specifying the workbook author}
 #'   \item{\bold{sheetName}}{ Name of the worksheet}
 #'   \item{\bold{gridLines}}{ A logical. If FALSE, the worksheet grid lines will be hidden.}
-#'   \item{\bold{startCol}}{ A vector specifiying the starting columns(s) to write df}
+#'   \item{\bold{startCol}}{ A vector specifiying the starting column(s) to write df}
 #'   \item{\bold{startRow}}{ A vector specifiying the starting row(s) to write df}
 #'   \item{\bold{xy}}{ An alternative to specifying startCol and startRow individually. 
 #'  A vector of the form c(startCol, startRow)}
@@ -43,33 +44,26 @@
 #' @examples
 #' 
 #' ## write to working directory
-#' write.xlsx(iris, file = "writeXLSX1.xlsx", colNames = TRUE, borders = "rows")
-#' write.xlsx(iris, file = "writeXLSX2.xlsx", colNames = TRUE, borders = "columns",
-#'  borderStyle = "dashed")
-#' 
 #' options("openxlsx.borderColour" = "#4F80BD") ## set default border colour
-#' write.xlsx(iris, file = "writeXLSX3.xlsx", colNames = TRUE, borders = "rows",
-#'  sheetName = "Iris data", gridLines = FALSE)
+#' write.xlsx(iris, file = "writeXLSX1.xlsx", colNames = TRUE, borders = "columns")
+#' write.xlsx(iris, file = "writeXLSX2.xlsx", colNames = TRUE, borders = "surrounding")
 #' 
-#' options("openxlsx.borderStyle" = "dashDot")
-#' write.xlsx(iris, file = "writeXLSX4.xlsx", colNames = TRUE, borders = "rows",
-#'  gridLines = FALSE)
 #' 
-#' options("openxlsx.borderStyle" = "medium")
 #' hs <- createStyle(textDecoration = "BOLD", fontColour = "#FFFFFF", fontSize=12,
-#'  fontName="Arial Narrow", fgFill = "#4F80BD")
-#'  
-#' write.xlsx(iris, file = "writeXLSX5.xlsx", colNames = TRUE, borders = "rows",
-#'  headerStyle = hs)
+#'                   fontName="Arial Narrow", fgFill = "#4F80BD")
 #' 
-#' ## Lists elements are written to individual worksheets
-#' l <- list(iris, mtcars, matrix(runif(1000), ncol = 5))
-#' write.xlsx(l, "writeList.xlsx")
-#' names(l) <- c("IRIS", "MTCARS", "RUNIF")
-#' write.xlsx(l, "writeList2.xlsx")
+#' write.xlsx(iris, file = "writeXLSX3.xlsx", colNames = TRUE, borders = "rows", headerStyle = hs)
+#' 
+#' ## Lists elements are written to individual worksheets, using list names as sheet names if available
+#' l <- list("IRIS" = iris, "MTCATS" = mtcars, matrix(runif(1000), ncol = 5))
+#' write.xlsx(l, "writeList1.xlsx")
+#' 
+#' ## different sheets can be given different parameters
+#' write.xlsx(l, "writeList2.xlsx", startCol = c(1,2,3), startRow = 2,
+#'            asTable = c(TRUE, TRUE, FALSE), withFilter = c(TRUE, FALSE, FALSE))
 #' 
 #' @export
-write.xlsx <- function(x, file, ...){
+write.xlsx <- function(x, file, asTable = FALSE, ...){
   
   
   ## set scientific notation penalty
@@ -96,8 +90,22 @@ write.xlsx <- function(x, file, ...){
   ## borderColour = "#4F81BD"
   ## borderStyle
   
+  #----writeDataTable---#
+  ## startCol = 1
+  ## startRow = 1
+  ## xy = NULL
+  ## colNames = TRUE
+  ## rowNames = FALSE
+  ## tableStyle = "TableStyleLight9"
+  ## tableName = NULL
+  ## headerStyle = NULL
+  ## withFilter = TRUE
+  
   #---saveWorkbook---#
-#   overwrite = TRUE
+  #   overwrite = TRUE
+  
+  if(!is.logical(asTable))
+    stop("asTable must be a logical.")
   
   creator <- ""
   if(creator %in% names(params))
@@ -114,7 +122,7 @@ write.xlsx <- function(x, file, ...){
   
   gridLines <- TRUE
   if("gridLines" %in% names(params)){
-    if(is.logical(params$gridLines)){
+    if(all(is.logical(params$gridLines))){
       gridLines <- params$gridLines
     }else{
       stop("Argument gridLines must be TRUE or FALSE")
@@ -130,10 +138,18 @@ write.xlsx <- function(x, file, ...){
     }
   }
   
+  withFilter <- TRUE
+  if("withFilter" %in% names(params)){
+    if(is.logical(params$withFilter)){
+      withFilter <- params$withFilter
+    }else{
+      stop("Argument withFilter must be TRUE or FALSE")
+    }
+  }
   
   startRow <- 1
   if("startRow" %in% names(params)){
-    if(startRow > 0){
+    if(all(startRow > 0)){
       startRow <- params$startRow
     }else{
       stop("startRow must be a positive integer")
@@ -142,14 +158,13 @@ write.xlsx <- function(x, file, ...){
   
   startCol <- 1
   if("startCol" %in% names(params)){
-    if(startCol > 0){
+    if(all(startCol > 0)){
       startCol <- params$startCol
     }else{
       stop("startCol must be a positive integer")
     }
   }
   
-  
   colNames <- TRUE
   if("colNames" %in% names(params)){
     if(is.logical(params$colNames)){
@@ -197,29 +212,43 @@ write.xlsx <- function(x, file, ...){
   
   headerStyle <- NULL
   if("headerStyle" %in% names(params)){
-    if("Style" %in% class(params$headerStyle)){
-      headerStyle <- params$headerStyle
+    
+    if(length(params$headerStyle) == 1){
+      if("Style" %in% class(params$headerStyle)){
+        headerStyle <- params$headerStyle
+      }else{
+        stop("headerStyle must be a style object.")
+      }
     }else{
-      stop("headerStyle must be a style object.")
+      if(all(sapply(params$headerStyle, function(x) "Style" %in% class(x)))){
+        headerStyle <- params$headerStyle
+      }else{
+        stop("headerStyle must be a style object.")
+      }
     }
   }
   
   borders <- NULL
   if("borders" %in% names(params)){
     borders <- tolower(params$borders)
-    if(!borders %in% c("surrounding", "rows", "columns"))
+    if(!all(borders %in% c("surrounding", "rows", "columns")))
       stop("Invalid borders argument")
   }
-
+  
   borderColour <- getOption("openxlsx.borderColour", "black")
   if("borderColour" %in% names(params))
     borderColour <- params$borderColour
   
   borderStyle <- getOption("openxlsx.borderStyle", "thin")
   if("borderStyle" %in% names(params)){
-    borderStyle <- validateBorderStyle(params$borderStyle)[[1]]
+    borderStyle <- validateBorderStyle(params$borderStyle)
   }
   
+  tableStyle <- "TableStyleLight9"
+  if("tableStyle" %in% names(params))
+    tableStyle <- params$tableStyle
+  
+  
   ## create new Workbook object
   wb <- Workbook$new(creator)
   
@@ -231,16 +260,112 @@ write.xlsx <- function(x, file, ...){
     
     if(is.null(nms)){
       nms <- paste("Sheet", 1:nSheets)
+    }else if(any("" %in% nms)){
+      nms[nms %in% ""] <- paste("Sheet", (1:nSheets)[nms %in% ""])
     }else{
       nms <- make.names(nms, unique  = TRUE)
     }
     
+    ## make all inputs as long as the list
+    if(length(withFilter) != nSheets)
+      withFilter <- rep_len(withFilter, length.out = nSheets)
+    
+    if(length(colNames) != nSheets)
+      colNames <- rep_len(colNames, length.out = nSheets)
+    
+    if(length(rowNames) != nSheets)
+      rowNames <- rep_len(rowNames, length.out = nSheets)
+    
+    if(length(startRow) != nSheets)
+      startRow <- rep_len(startRow, length.out = nSheets)
+    
+    if(length(startCol) != nSheets)
+      startCol <- rep_len(startCol, length.out = nSheets)
+        
+    if(length(headerStyle) != nSheets & !is.null(headerStyle))
+      headerStyle <- rep_len(headerStyle, length.out = nSheets)
+    
+    if(length(borders) != nSheets & !is.null(borders))
+      borders <- rep_len(borders, length.out = nSheets)
+    
+    if(length(borderColour) != nSheets)
+      borderColour <- rep_len(borderColour, length.out = nSheets)
+    
+    if(length(borderStyle) != nSheets)
+      borderStyle <- rep_len(borderStyle, length.out = nSheets)
+    
+    if(length(asTable) != nSheets)
+      asTable <- rep_len(asTable, length.out = nSheets)
+    
+    if(length(tableStyle) != nSheets)
+      tableStyle <- rep_len(tableStyle, length.out = nSheets)
+    
     for(i in 1:nSheets){
       
       wb$addWorksheet(nms[[i]], showGridLines = gridLines)
+      
+      if(asTable[i]){
+                
+        writeDataTable(wb = wb,
+                       sheet = i,
+                       x = x[[i]],
+                       startCol = startCol[i],
+                       startRow = startRow[i],
+                       xy = xy,
+                       colNames = colNames[i],
+                       rowNames = rowNames[i],
+                       tableStyle = tableStyle[i],
+                       tableName = NULL,
+                       headerStyle = headerStyle[i],
+                       withFilter = withFilter[i])
+        
+      }else{
+        
+        writeData(wb = wb, 
+                  sheet = i,
+                  x = x[[i]],
+                  startCol = startCol[i],
+                  startRow = startRow[i], 
+                  xy = xy,
+                  colNames = colNames[i],
+                  rowNames = rowNames[i],
+                  headerStyle = headerStyle[i],
+                  borders = borders[i],
+                  borderColour = borderColour[i],
+                  borderStyle = borderStyle[i])
+        
+      }
+      
+      
+    }
+    
+    
+  }else{
+    
+    wb$addWorksheet(sheetName, showGridLines = gridLines)
+    
+    if(asTable){
+      
+      if(!"data.frame" %in% class(x))
+        stop("x must be a data.frame is asTable == TRUE")
+      
+      writeDataTable(wb = wb,
+                     sheet = 1,
+                     x = x,
+                     startCol = startCol,
+                     startRow = startRow,
+                     xy = xy,
+                     colNames = colNames,
+                     rowNames = rowNames,
+                     tableStyle = tableStyle,
+                     tableName = NULL,
+                     headerStyle = headerStyle)
+      
+    }else{
+            
       writeData(wb = wb, 
-                sheet = i,
-                x = x[[i]],
+                sheet = 1,
+                x = x,
                 startCol = startCol,
                 startRow = startRow, 
                 xy = xy,
@@ -249,31 +374,11 @@ write.xlsx <- function(x, file, ...){
                 headerStyle = headerStyle,
                 borders = borders,
                 borderColour = borderColour,
-                borderStyle = borderStyle)
-      
-      
+                borderStyle = borderStyle)      
     }
-  
-    
-  }else{
-    
-    wb$addWorksheet(sheetName, showGridLines = gridLines)
-    
-    writeData(wb = wb, 
-              sheet = 1,
-              x = x,
-              startCol = startCol,
-              startRow = startRow, 
-              xy = xy,
-              colNames = colNames,
-              rowNames = rowNames,
-              headerStyle = headerStyle,
-              borders = borders,
-              borderColour = borderColour,
-              borderStyle = borderStyle)
     
   }
-
+  
   
   
   saveWorkbook(wb = wb, file = file, overwrite = overwrite)
